/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/*
 * Caja
 *
 * Copyright (C) 2000 Eazel, Inc.
 *
 * Caja is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Caja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Authors: Darin Adler <darin@bentspoon.com>
 */

#include <config.h>
#include "caja-desktop-window.h"
#include "caja-window-private.h"
#include "caja-actions.h"

#include <X11/Xatom.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <eel/eel-background.h>
#include <eel/eel-vfs-extensions.h>
#include <libcaja-private/caja-file-utilities.h>
#include <libcaja-private/caja-icon-names.h>
#include <libcaja-private/caja-global-preferences.h>
#include <gio/gio.h>
#include <glib/gi18n.h>    
#include <json-glib/json-glib.h>

#include "kiran-license-dbus-helper-generated.h"

/**
 * 当前系统授权状态
 */
typedef enum
{
    LICENSE_STATUS_UNREGISTERED = 0,     /**< 试用期内 */
    LICENSE_STATUS_REGISTERED,           /**< 已授权，在服务期内 */
    LICENSE_STATUS_NOREQUIRED,           /**< 不需要授权 */
    LICENSE_STATUS_REGISTERED_EXPIRED,   /**< 授权过期 */
    LICENSE_STATUS_UNREGISTERED_EXPIRED, /**< 试用期过期 */
    LICENSE_STATUS_UNKNOWN               /**< 授权状态未知 */
} KiranLicenseStatus;

/* Tell screen readers that this is a desktop window */

G_DEFINE_TYPE (CajaDesktopWindowAccessible, caja_desktop_window_accessible,
               GTK_TYPE_WINDOW_ACCESSIBLE);

static AtkAttributeSet *
desktop_get_attributes (AtkObject *accessible)
{
    AtkAttributeSet *attributes;
    AtkAttribute *is_desktop;

    attributes = ATK_OBJECT_CLASS (caja_desktop_window_accessible_parent_class)->get_attributes (accessible);

    is_desktop = g_malloc (sizeof (AtkAttribute));
    is_desktop->name = g_strdup ("is-desktop");
    is_desktop->value = g_strdup ("true");

    attributes = g_slist_append (attributes, is_desktop);

    return attributes;
}

static void
caja_desktop_window_accessible_init (CajaDesktopWindowAccessible *window)
{
}

static void
caja_desktop_window_accessible_class_init (CajaDesktopWindowAccessibleClass *klass)
{
    AtkObjectClass *aclass = ATK_OBJECT_CLASS (klass);

    aclass->get_attributes = desktop_get_attributes;
}

struct _CajaDesktopWindowPrivate
{
    gulong size_changed_id;

    gboolean loaded;
    gboolean desktop_is_home;

    GCancellable  *cancellable;
    gchar         *bus_name;
    gchar         *object_path;
    guint          name_id;

    KiranLicenseHelper *proxy;
    KiranLicenseStatus license_status; /* 系统激活状态 */
};

G_DEFINE_TYPE_WITH_PRIVATE (CajaDesktopWindow, caja_desktop_window,
               CAJA_TYPE_SPATIAL_WINDOW);

static void
desktop_directory_changed_callback (gpointer data)
{
    CajaDesktopWindow *window = CAJA_DESKTOP_WINDOW (data);
    gboolean desktop_is_home = g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR);

    if (window->details->desktop_is_home == desktop_is_home) //the settings value not change
        return;

    window->details->desktop_is_home = desktop_is_home;
    caja_window_reload (CAJA_WINDOW (data));
}

static void
handle_license_info(CajaDesktopWindow *window, gchar *license_info)
{
    JsonParser *parser;
    JsonNode *root_node;
    JsonObject *json_object;
    KiranLicenseStatus status;
    guint32 expire_time;
    gboolean expired;
    gboolean ret;

    if (!license_info)
	return;

    parser = json_parser_new ();
    
    ret = json_parser_load_from_data(parser,
		                     license_info,
				     -1,
				     NULL);

    if (!ret)
    {
	g_object_unref (parser);
	return;
    }

    root_node = json_parser_get_root (parser);
    json_object = json_node_get_object (root_node);
    status = json_object_get_int_member(json_object, "license_status");
    expire_time = json_object_get_int_member(json_object, "expired_time");

    expired = (expire_time < time(NULL));
    if (expired)
    {
        switch (status)
        {
        case LICENSE_STATUS_UNREGISTERED:
            status = LICENSE_STATUS_UNREGISTERED_EXPIRED;
            break;
        case LICENSE_STATUS_REGISTERED:
            status = LICENSE_STATUS_REGISTERED_EXPIRED;
            break;
        default:
            break;
        }
    }

    window->details->license_status = status;
    g_object_unref (parser);
}

static void 
get_license_json_cb (GObject      *source_object,
                     GAsyncResult *res,
                     gpointer     user_data)
{
    CajaDesktopWindow *window;
    GError *error;
    gchar **strokes;

    window = CAJA_DESKTOP_WINDOW (user_data);

    error = NULL;
    strokes = NULL;
    kiran_license_helper_call_get_license_json_finish (KIRAN_LICENSE_HELPER (source_object),
                                                       &strokes, res, &error);

    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
    {
        g_error_free (error);
        return;
    }

    if (error != NULL)
    {
        g_warning ("%s", error->message);
        g_error_free (error);
        return;
    }

    handle_license_info(window, strokes);

    gtk_widget_queue_draw (GTK_WIDGET (window));
}

static void
license_changed_cb(KiranLicenseHelper *proxy,
		   gboolean     status_changed,
		   gpointer     user_data)
{
    CajaDesktopWindow *window;

    window = CAJA_DESKTOP_WINDOW (user_data);

    if (status_changed)
    {
        kiran_license_helper_call_get_license_json (window->details->proxy,
                                                    window->details->cancellable,
                                                    get_license_json_cb,
                                                    window);
    }
}

static void
proxy_ready_cb (GObject      *source_object,
                GAsyncResult *res,
                gpointer      user_data)
{
    CajaDesktopWindow *window;
    KiranLicenseHelper *proxy;
    GError *error;

    window = CAJA_DESKTOP_WINDOW (user_data);

    error = NULL;
    proxy = kiran_license_helper_proxy_new_finish (res, &error);

    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
    {
        g_error_free (error);
        return;
    }

    window->details->proxy = proxy;

    if (error)
    {
        g_warning ("%s", error->message);
        g_error_free (error);
        return;
    }

    g_signal_connect(proxy,
                     "license-changed",
                     G_CALLBACK (license_changed_cb), window);

    kiran_license_helper_call_get_license_json (window->details->proxy,
                                                window->details->cancellable,
                                                get_license_json_cb,
                                                window);
}

static void
name_appeared_cb (GDBusConnection *connection,
                  const gchar     *name,
                  const gchar     *name_owner,
                  gpointer         user_data)
{
    CajaDesktopWindow *window;

    window = CAJA_DESKTOP_WINDOW (user_data);

    kiran_license_helper_proxy_new (connection, G_DBUS_PROXY_FLAGS_NONE,
                                    window->details->bus_name, window->details->object_path,
                                    window->details->cancellable, proxy_ready_cb, window);
}

static void
name_vanished_cb (GDBusConnection *connection,
                  const gchar     *name,
                  gpointer         user_data)
{
    CajaDesktopWindow *window;

    window = CAJA_DESKTOP_WINDOW (user_data);

    g_clear_object (&window->details->proxy);
}

static void
caja_desktop_window_init (CajaDesktopWindow *window)
{
    GtkAction *action;
    AtkObject *accessible;

    window->details = caja_desktop_window_get_instance_private (window);

    GtkStyleContext *context;

    context = gtk_widget_get_style_context (GTK_WIDGET (window));
    gtk_style_context_add_class (context, "caja-desktop-window");

    gtk_window_move (GTK_WINDOW (window), 0, 0);

    /* shouldn't really be needed given our semantic type
     * of _NET_WM_TYPE_DESKTOP, but why not
     */
    gtk_window_set_resizable (GTK_WINDOW (window),
                              FALSE);

    g_object_set_data (G_OBJECT (window), "is_desktop_window",
                       GINT_TO_POINTER (1));

    gtk_widget_hide (CAJA_WINDOW (window)->details->statusbar);
    gtk_widget_hide (CAJA_WINDOW (window)->details->menubar);

    /* Don't allow close action on desktop */
    action = gtk_action_group_get_action (CAJA_WINDOW (window)->details->main_action_group,
                                          CAJA_ACTION_CLOSE);
    gtk_action_set_sensitive (action, FALSE);

    /* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
    accessible = gtk_widget_get_accessible (GTK_WIDGET (window));

    if (accessible) {
        atk_object_set_name (accessible, _("Desktop"));
    }

    window->details->desktop_is_home = g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR);
    g_signal_connect_swapped (caja_preferences, "changed::" CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR,
                              G_CALLBACK(desktop_directory_changed_callback),
                              window);

    window->details->license_status = LICENSE_STATUS_UNKNOWN;
    window->details->cancellable = g_cancellable_new ();
    window->details->bus_name =g_strdup ("com.kylinsec.Kiran.LicenseHelper");
    window->details->object_path =g_strdup ("/com/kylinsec/Kiran/LicenseHelper");
    window->details->name_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM, window->details->bus_name,
                                                 G_BUS_NAME_WATCHER_FLAGS_AUTO_START,
                                                 name_appeared_cb, name_vanished_cb,
                                                 window, NULL);
}

static gint
caja_desktop_window_delete_event (CajaDesktopWindow *window)
{
    /* Returning true tells GTK+ not to delete the window. */
    return TRUE;
}

void
caja_desktop_window_update_directory (CajaDesktopWindow *window)
{
    GFile *location;

    g_assert (CAJA_IS_DESKTOP_WINDOW (window));

    location = g_file_new_for_uri (EEL_DESKTOP_URI);
    caja_window_go_to (CAJA_WINDOW (window), location);
    window->details->loaded = TRUE;

    g_object_unref (location);
}

static void
caja_desktop_window_screen_size_changed (GdkScreen             *screen,
        CajaDesktopWindow *window)
{
    int width_request, height_request;
    int scale;

    scale = gdk_window_get_scale_factor (gdk_screen_get_root_window (screen));

    width_request = WidthOfScreen (gdk_x11_screen_get_xscreen (screen)) / scale;
    height_request = HeightOfScreen (gdk_x11_screen_get_xscreen (screen)) / scale;

    g_object_set (window,
                  "width_request", width_request,
                  "height_request", height_request,
                  NULL);
}

CajaDesktopWindow *
caja_desktop_window_new (CajaApplication *application,
                         GdkScreen           *screen)
{
    CajaDesktopWindow *window;
    int width_request, height_request;
    int scale;

    scale = gdk_window_get_scale_factor (gdk_screen_get_root_window (screen));

    width_request = WidthOfScreen (gdk_x11_screen_get_xscreen (screen)) / scale;
    height_request = HeightOfScreen (gdk_x11_screen_get_xscreen (screen)) / scale;

    window = CAJA_DESKTOP_WINDOW
             (gtk_widget_new (caja_desktop_window_get_type(),
                              "app", application,
                              "width_request", width_request,
                              "height_request", height_request,
                              "screen", screen,
                              NULL));
    /* Stop wrong desktop window size in GTK 3.20*/
    /* We don't want to set a default size, which the parent does, since this */
    /* will cause the desktop window to open at the wrong size in gtk 3.20 */
    gtk_window_set_default_size (GTK_WINDOW (window), -1, -1);

    /* Special sawmill setting*/
    GdkWindow *gdkwin;
    gtk_widget_realize (GTK_WIDGET (window));
    gdkwin = gtk_widget_get_window (GTK_WIDGET (window));
    if (gdk_window_ensure_native (gdkwin)) {
        Display *disp = GDK_DISPLAY_XDISPLAY (gdk_window_get_display (gdkwin));
        XClassHint *xch = XAllocClassHint ();
        xch->res_name = "desktop_window";
        xch->res_class = "Caja";
        XSetClassHint (disp, GDK_WINDOW_XID(gdkwin), xch);
        XFree(xch);
    }

    g_signal_connect (window, "delete_event", G_CALLBACK (caja_desktop_window_delete_event), NULL);

    /* Point window at the desktop folder.
     * Note that caja_desktop_window_init is too early to do this.
     */
    caja_desktop_window_update_directory (window);

    return window;
}

static void
map (GtkWidget *widget)
{
    /* Chain up to realize our children */
    GTK_WIDGET_CLASS (caja_desktop_window_parent_class)->map (widget);
    gdk_window_lower (gtk_widget_get_window (widget));
}

static void
unrealize (GtkWidget *widget)
{
    CajaDesktopWindow *window;
    CajaDesktopWindowPrivate *details;
    GdkWindow *root_window;

    window = CAJA_DESKTOP_WINDOW (widget);
    details = window->details;

    root_window = gdk_screen_get_root_window (
                      gtk_window_get_screen (GTK_WINDOW (window)));

    gdk_property_delete (root_window,
                         gdk_atom_intern ("CAJA_DESKTOP_WINDOW_ID", TRUE));

    if (details->size_changed_id != 0) {
        g_signal_handler_disconnect (gtk_window_get_screen (GTK_WINDOW (window)),
                         details->size_changed_id);
        details->size_changed_id = 0;
    }

    GTK_WIDGET_CLASS (caja_desktop_window_parent_class)->unrealize (widget);
}

static void
set_wmspec_desktop_hint (GdkWindow *window)
{
    GdkAtom atom;

    atom = gdk_atom_intern ("_NET_WM_WINDOW_TYPE_DESKTOP", FALSE);

    gdk_property_change (window,
                         gdk_atom_intern ("_NET_WM_WINDOW_TYPE", FALSE),
                         gdk_x11_xatom_to_atom (XA_ATOM), 32,
                         GDK_PROP_MODE_REPLACE, (guchar *) &atom, 1);
}

static void
set_desktop_window_id (CajaDesktopWindow *window,
                       GdkWindow             *gdkwindow)
{
    /* Tuck the desktop windows xid in the root to indicate we own the desktop.
     */
    Window window_xid;
    GdkWindow *root_window;

    root_window = gdk_screen_get_root_window (
                      gtk_window_get_screen (GTK_WINDOW (window)));

    window_xid = GDK_WINDOW_XID (gdkwindow);

    gdk_property_change (root_window,
                         gdk_atom_intern ("CAJA_DESKTOP_WINDOW_ID", FALSE),
                         gdk_x11_xatom_to_atom (XA_WINDOW), 32,
                         GDK_PROP_MODE_REPLACE, (guchar *) &window_xid, 1);
}

static void
realize (GtkWidget *widget)
{
    CajaDesktopWindow *window;
    CajaDesktopWindowPrivate *details;
    window = CAJA_DESKTOP_WINDOW (widget);
    details = window->details;

    /* Make sure we get keyboard events */
    gtk_widget_set_events (widget, gtk_widget_get_events (widget)
                           | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
    /* Do the work of realizing. */
    GTK_WIDGET_CLASS (caja_desktop_window_parent_class)->realize (widget);

    /* This is the new way to set up the desktop window */
    set_wmspec_desktop_hint (gtk_widget_get_window (widget));

    set_desktop_window_id (window, gtk_widget_get_window (widget));

    details->size_changed_id =
        g_signal_connect (gtk_window_get_screen (GTK_WINDOW (window)), "size_changed",
                          G_CALLBACK (caja_desktop_window_screen_size_changed), window);
}

static void 
draw_system_license_status_tip (GtkWidget *widget,
		                cairo_t   *cr)
{
    CajaDesktopWindow *window;
    CajaDesktopWindowPrivate *details;
    GtkAllocation allocation;
    PangoLayout *layout;
    PangoLayout *second_layout;
    GtkStyleContext *context;
    PangoFontDescription *desc;
    PangoFontDescription *second_desc;
    gint width, height;
    gint text_width, text_height;
    gint second_text_width, second_text_height;
    gchar *buffer; 
    gchar *second_buffer;
    gint right, bottom, space;
    gint w, h;
    gboolean need_draw_second;
    GtkBorder margin;
    GdkDisplay *display;
    GdkMonitor *monitor;
    GdkRectangle geometry;

    window = CAJA_DESKTOP_WINDOW (widget);
    details = window->details;

    need_draw_second = TRUE;
    context = gtk_widget_get_style_context (widget);

    gtk_style_context_save (context);
    cairo_save (cr);

    gtk_style_context_add_class (context, "license-status-text");
    gtk_widget_get_allocation (widget, &allocation);
    gtk_style_context_get_margin (context, gtk_widget_get_state_flags(widget), &margin);

    right = margin.right;
    bottom = margin.bottom;
    space = margin.top;

    display = gdk_display_get_default ();
    monitor = gdk_display_get_primary_monitor (display);
    gdk_monitor_get_geometry (monitor, &geometry);
    width = geometry.x + geometry.width;
    height = geometry.y + geometry.height;

    buffer = NULL;
    second_buffer = NULL;

    switch (details->license_status)
    {
	case LICENSE_STATUS_REGISTERED:
	case LICENSE_STATUS_NOREQUIRED:
	    buffer = g_strdup (_("KylinSec"));
	    need_draw_second = FALSE; 
	    break;

	case LICENSE_STATUS_REGISTERED_EXPIRED:
            buffer = g_strdup (_("KylinSec"));
	    break;

	case LICENSE_STATUS_UNREGISTERED:
            buffer = g_strdup (_("KylinSec(not activated)"));
	    need_draw_second = FALSE; 
	    break;

	case LICENSE_STATUS_UNREGISTERED_EXPIRED:
            buffer = g_strdup (_("KylinSec(not activated)"));
            second_buffer = g_strdup (_("Please move to Control center to \nactivate your operaing system"));
	    break;

	default:
            buffer = g_strdup (_("KylinSec"));
            second_buffer = g_strdup (_("Activation status is abnormal"));
	    break;
    }

    gtk_style_context_add_class (context, "text-primary");
    layout = pango_cairo_create_layout(cr);
    desc = gtk_style_context_get_font(context, gtk_widget_get_state_flags(widget));
    pango_layout_set_font_description(layout, desc);
    pango_layout_set_text (layout, buffer, -1);
    pango_layout_get_pixel_size (layout, &text_width, &text_height); 

    gtk_style_context_remove_class (context, "text-primary");
    gtk_style_context_add_class (context, "text-secondary");
    second_layout = pango_cairo_create_layout(cr);
    desc = gtk_style_context_get_font(context, gtk_widget_get_state_flags(widget));
    pango_layout_set_font_description(second_layout, desc);

    if (need_draw_second)
        pango_layout_set_text (second_layout, second_buffer, -1);

    pango_layout_get_pixel_size (second_layout, &second_text_width, &second_text_height); 

    if (need_draw_second)
    {
        if (text_width > second_text_width)
            w = width - text_width;
        else
            w = width - second_text_width;

        w = w - bottom;
        h = height - (text_height + second_text_height + space + bottom);
    }
    else
    {
	w = width - (text_width + bottom);
	h = height - (text_height + bottom);
    }

    gtk_render_layout (context, cr, 
		       w, 
		       h, layout);

    if (need_draw_second)
    {
        h = height - (second_text_height + bottom);
        gtk_render_layout (context, cr, 
            	       w, 
            	       h, second_layout);
    }

    g_object_unref (layout);
    g_object_unref (second_layout);
    g_free (buffer);
    g_free (second_buffer);

    gtk_style_context_restore (context);
    cairo_restore (cr);
}

static gboolean
draw (GtkWidget *widget,
      cairo_t   *cr)
{
    eel_background_draw (widget, cr);

    draw_system_license_status_tip (widget, cr); /* 绘制系统激活信息 */

    return GTK_WIDGET_CLASS (caja_desktop_window_parent_class)->draw (widget, cr);
}

static char *
real_get_title (CajaWindow *window)
{
    return g_strdup (_("Desktop"));
}

static CajaIconInfo *
real_get_icon (CajaWindow *window,
               CajaWindowSlot *slot)
{
    gint scale = gtk_widget_get_scale_factor (GTK_WIDGET (window));
    return caja_icon_info_lookup_from_name (CAJA_ICON_DESKTOP, 48, scale);
}

static void
caja_desktop_window_finalize (GObject *object)
{
    CajaDesktopWindow *window;

    window = CAJA_DESKTOP_WINDOW (object);
    g_signal_handlers_disconnect_by_func (caja_preferences,
                                          desktop_directory_changed_callback,
                                          window);

    if (window->details->name_id > 0)
    {
          g_bus_unwatch_name (window->details->name_id);
          window->details->name_id = 0;
    }

    g_cancellable_cancel (window->details->cancellable);
    g_clear_object (&window->details->cancellable);
    g_clear_object (&window->details->proxy);
    g_free (window->details->bus_name);
    g_free (window->details->object_path);

    G_OBJECT_CLASS (caja_desktop_window_parent_class)->finalize (object);
}

static void
caja_desktop_window_class_init (CajaDesktopWindowClass *klass)
{
    GtkWidgetClass *wclass = GTK_WIDGET_CLASS (klass);
    CajaWindowClass *nclass = CAJA_WINDOW_CLASS (klass);


    G_OBJECT_CLASS (klass)->finalize = caja_desktop_window_finalize;

    wclass->realize = realize;
    wclass->unrealize = unrealize;
    wclass->map = map;
    wclass->draw = draw;

    gtk_widget_class_set_accessible_type (wclass, CAJA_TYPE_DESKTOP_WINDOW_ACCESSIBLE);

    nclass->window_type = CAJA_WINDOW_DESKTOP;
    nclass->get_title = real_get_title;
    nclass->get_icon = real_get_icon;
}

gboolean
caja_desktop_window_loaded (CajaDesktopWindow *window)
{
    return window->details->loaded;
}
